% One sector general equilibrium menu cost model
%
% The state vector is three dimensional:
% a: idiosyncratic productivity
% rp: real price
% rad: real aggregate demand
%
% We vectorize it using the reshape command. The order of the state 
% variables when they are reshaped is (a, rp, rad)
%
% Jon Steinsson and Emi Nakamura, August 2006
%**************************************************************

clear;
tic

%% Set Parameters
%**************************************************************

theta = 4;    % Elasticity of demand
psi = 0;     % One over the Frisch elasticity of labor supply
gamma = 1;   % coefficient of relative risk aversion
FlexSSL = (theta-1)/theta; % Steady State labor under flexible prices
beta = 0.96^(1/12);   % Discount factor

% Parameters of the process followed by nominal aggregate demand
% log(S_t) - log(S_t-1) = mu + nu_t
% where nu_t ~ N(0,sigma_eta^2), i.i.d.
%  mu = 0.02/12;
%  sigma_eta = 0.0037;
% mu = 0.0022 ;
mu = 0.00;
sigma_eta = 0.0039 ;

% Estimated process for nominal aggregate demand

%importfile1('ngr.xls'); 
%QGR = Sheet1/100;
%duration = length(QGR);

% figure(1)
% hist(QGR,floor(duration/2))

%mu_emp = mean(QGR)/4;
%sigma_eta_emp = sqrt(var(QGR)/4);

%[mu sigma_eta]
%[mu_emp sigma_eta_emp]

 % mu = mu_emp;
 % sigma_eta = sigma_eta_emp;

% Parameters of the idiosyncratic productivity process (log A_it)
% log(A_it) = rho*log(A_it-1) + eps_it
% where eps_it ~ N(0,sigma_eps^2), i.i.d. 
% The process is truncated at +/- trunc_eps multiples of the unconditional 
% standard deviation of log(A) 
rho = 0.7;
% sigma_eps = 0.0425;
sigma_eps = 0.5*0.0369;

sm = 0.0;  % Intermediate input share in the production function
K = 0.00488 ;% K = 0.027; % Menu cost (Notice: In the notation of the paper this is "K/Y_ss"
           % while menu cost reported in table 6-9 in the paper is 
           % "((theta-1)/theta)*(K/Y_ss)". So the numbers in the tables
           % should be multiplied by theta/(theta-1) when plugged in here
           % to replicate the results in the paper.)

% Location of the middle of the rad grid relative to the no-uncertainty 
% steady state value of rad
rad_shift = 0.5*mu+0.003;

% Span of rp grid relative to a grid
rp_span = 1.3;

% The slope and level of the initial guess for the function G function
% i.e. the function Gamma in our Multi-Sector Menu Cost paper.
% The initial guess is a step function. "slope" gives the number of grid
% points on the rad grid between steps in this step function. This means
% that the slope is greater for lower values of "slope".
% If levelshift = 0 the Gamma function take a value of zero (in logs) in 
% the middle of the grid. More generally, if levelshift = N the Gamma
% function takes a value N gridpoints above zero in the middle of the grid.
slope = 3; 
levelshift = floor(12*mu/0.02);

% Number of points on the grid for each of the three state variables
agridsize = 40;
rpgridsize = 400;
radgridsize = floor(25*rpgridsize/200);
% The following values:
% agridsize = 40 which gives ainc = 0.0091
% rpgridsize = 100 which gives rpinc = 0.0047
% radgridsize = 12.
% yield reliable results for price change stats (freq, size, etc) 
% when the model is calibrated as follow:
% theta=4, psi=0, gamma=1,
% sigma_eta = 0.0037; mu = 0.002;
% rho = 0.7, sigma_eps = 0.0425;
% sm = 0, K = 0.027.
% Note: To get reliable result for stdOutput one needs roughly
% rpgridsize = 500 and radgridsize = floor(10*rpgridsize/100);
% radgridsize needs to expand proportionately with rpgridsize
% sm > 0 implies one needs a larger radgridsize

% radgridsize needs to expand proportionately with rpgridsize
% sm > 0 implies one needs a larger radgridsize

% Maximum number of iterations on G
MaxItOnG = 100;

Ssize = agridsize*rpgridsize*radgridsize;

%% Solve for the flexible price steady state 
%****************************************************************

par1 = sm*(theta-1)/(theta-sm*(theta-1));

FlexSSC = FlexSSL*par1^(sm/(1-sm))*(1+par1)^(-1/(1-sm));
FlexSSM = par1*FlexSSC;
FlexSSY = FlexSSC + FlexSSM;


omega = (theta-1)/theta*(1-sm)*(1+par1)*FlexSSL^(-psi-1)*FlexSSC^(1-gamma); % Level parameter on disutility of labor
omega2 = omega*(FlexSSL)^psi; % Marginal disutility of changing prices

K = K*FlexSSY;

FlexSSL = log(FlexSSL);
FlexSSC = log(FlexSSC);
if sm > 0
    FlexSSM = log(FlexSSM);
end
FlexSSY = log(FlexSSY);

%% Print Out Parameters
%****************************************************************

fprintf('\n')
fprintf('theta:    %5.5f\n',theta)
fprintf('psi:      %5.5f\n',psi)
fprintf('gamma:    %5.5f\n',gamma)
fprintf('sm:       %5.5f\n',sm)
fprintf('K:        %5.5f\n',K)
fprintf('rho:      %5.5f\n',rho)
fprintf('sigma:    %5.5f\n',sigma_eps)
fprintf('FlexSSC:  %5.5f\n',exp(FlexSSC))
fprintf('K/FlexSSY:%5.5f\n\n',K/exp(FlexSSY))
fprintf('Size of State Space: %10.0f\n\n',Ssize)

%% Set grid parameters
%****************************************************************

if agridsize < 30
    error('agridsize < 30: Results unreliable!')
end

trunc_eps = 3;
amax = trunc_eps*(sigma_eps^2/(1-rho^2))^(1/2);
amin = -trunc_eps*(sigma_eps^2/(1-rho^2))^(1/2);
ainc = (amax-amin)/(agridsize-1) - mod((amax-amin)/(agridsize-1),10^(-7));
amin = amin - mod(amin,10^(-7));
amax = amin + (agridsize-1)*ainc;

% rp denotes the real price
% np denotes the nominal price
% The rp grid is a grid for the log of the real price
rpmax = -amin*rp_span;
rpmin = -amax*rp_span;
if rpmax < 0.15
    rpmax = 0.15;
    rpmin = -0.15;
end
rpinc = (rpmax-rpmin)/(rpgridsize-1) - mod((rpmax-rpmin)/(rpgridsize-1),10^(-7));
rpmin = rpmin - mod(rpmin,10^(-7));
rpmax = rpmin + (rpgridsize-1)*rpinc;

if rpinc > 0.0075
    error('rpinc > 0.0075: Results unreliable')
end

radmin = FlexSSC - radgridsize/2*rpinc + rad_shift;

%% Create the real price and idiosyncratic productivity vectors
%**************************************************************

a = amin:ainc:amax; a = a';

rp = zeros(rpgridsize,1);
rp(1) = rpmin;
for ii = 2:rpgridsize
    rp(ii) = rp(ii-1) + rpinc;
end

rad = zeros(radgridsize,1);
rad(1) = radmin;
for ii = 2:radgridsize
    rad(ii) = rad(ii-1) + rpinc;
end

% Create Probability Distribution for the idiosyncratic shock
%**************************************************************
% Proba is a matrix that gives transition probabilities of going
% from state a to state a', i.e. Proba(i,j) = Prob(a' = j | a = i)
% 
% The procedure of Tauchen (1986) is used to approximate the 
% AR(1) process for log(A_it)

Proba = CreateProba(rho,sigma_eps,a);

%fprintf('Proba(1:8,1:8) ')
%Proba(1:8,1:8)

% Create probability distribution for nominal aggregate demand
%***************************************************************

ProbNgr = CreateProbNgr(mu,sigma_eta,rad);

%fprintf('ProbaNgr(1:8,1:8) ')
%ProbNgr(1:8,1:8)

% Guess a matrix G which gives the log inflation rate given S_t/P_t-1
%**************************************************************

G = zeros(radgridsize,1);
%G = (rp(2)-rp(1))*ones(radgridsize,1);
G(1,1) = -(radgridsize/(2*slope)-mod(radgridsize/(2*slope),1))*(rp(2)-rp(1)) ...
    + levelshift*(rp(2)-rp(1));
for ii = 2:radgridsize
    if mod(ii,slope) == 0
        G(ii,1) = G(ii-1,1) + rp(2) - rp(1);
    else
        G(ii,1) = G(ii-1,1);
    end
end
%figure(99)
%plot(rad,G)

%% Calculate the profit function
%**************************************************************

par1 = psi+1;
par2 = gamma;
par3 = exp(FlexSSC)/exp(FlexSSY);
par4 = exp(FlexSSM)/exp(FlexSSY) - sm;
par5 = 1-sm;

a3d = repmat(a,[1 rpgridsize radgridsize]);

rp3d = repmat(rp,[1 agridsize radgridsize]);
rp3d = permute(rp3d, [2 1 3]);

rad3d = repmat(rad,[1 agridsize rpgridsize]);
rad3d = permute(rad3d,[2 3 1]);

if sm > 0
    l3d = FlexSSL + (par3+par2*par4)/(par5-par1*par4)*(rad3d-FlexSSC);
    m3d = FlexSSM + par1*(l3d - FlexSSL) + par2*(rad3d-FlexSSC);
else
    l3d = FlexSSL + (rad3d-FlexSSC);
    m3d = 0*rad3d;
end

rPi3d = exp(rad3d + m3d).*exp(rp3d).^(1-theta) ...
    - (1-sm)^(sm-1)*sm^(-sm)*omega^(1-sm)*exp(l3d).^(psi*(1-sm)).*exp(rad3d).^(gamma*(1-sm)).*exp(-a3d).*exp(rad3d+m3d).*exp(rp3d).^(-theta);
rPi = reshape(rPi3d,[Ssize 1]);

menucost3d = omega2*K*exp(rad3d).^gamma;
menucost = reshape(menucost3d,[Ssize 1]);

clear a3d rp3d rad3d rPi3d menucost3d l3d m3d

% figure(4)
% rPi_plot = reshape(rPi,[agridsize rpgridsize radgridsize]);
% rPi_plot = squeeze(rPi_plot(:,:,1));
% surf(a,rp,rPi_plot')
% title('Profit Function')
% xlabel('a')
% ylabel('rp')

% Calculate the ratio of marginal utility
%**************************************************************
% The (i,j)th element of RMU is the ratio of marginal utility in 
% state j of period t+1 to the marginal utility in state i of
% period t.

RMU = ((1./exp(rad))*(exp(rad)')).^(-gamma);

% Initialize ErV matrix (Expected Value next period)
%**************************************************************

ErV = 1.5*ones(Ssize, 1)*max(0,mean(rPi)/(1-beta));
rPi3d = reshape(rPi,[agridsize rpgridsize radgridsize]);
tolV = 0.001*rPi3d(floor(agridsize/2),floor(rpgridsize/2),floor(radgridsize/2))/(1-beta);
clear rPi3d

fprintf('tolV:     %10.6f\n',tolV)

% Initialize stationary distribution Q
%**************************************************************

Qinit = ones(Ssize,1)/Ssize;
tolQ = Qinit(1)/150;

%% Solve for the Equilibrium
%**************************************************************

[G,rV,F,F_c,Q] = EqSolverGE1Basic(rPi,menucost,RMU,beta,theta,a,rp,rad,...
    Proba,ProbNgr,ErV,Qinit,G,tolV,tolQ,MaxItOnG,1);

GError = CalcGError(G,F,Q,a,rp,rad,theta,Proba,ProbNgr);

PCStats = RigidityStats(Q,F,a,rp,rad,theta,FlexSSC,menucost);

fprintf('\n')
fprintf('Frequency of Price Change:   %5.3f\n',PCStats.freq)
fprintf('Frac Up:                     %5.3f\n',PCStats.fracup)
fprintf('Size of Price Changes:       %5.3f\n',PCStats.abssizepc)
fprintf('Size of Price Increases:     %5.3f\n',PCStats.sizeup)
fprintf('Size of Price Decreases:     %5.3f\n',PCStats.sizedw)
fprintf('Av. Relative Price:          %5.5f\n\n',PCStats.meanp)
fprintf('Labor Productivity:          %5.5f\n',PCStats.lprod)
fprintf('Labor Productivity (2nd):    %5.5f\n',PCStats.lprod2nd)
fprintf('Standard Deviation of log Output:%5.5f\n',PCStats.stdLogOutput)
fprintf('Average of Log Output:           %5.5f\n',PCStats.meanLogOutput)
fprintf('Labor Prod Flex:             %5.5f\n',PCStats.lprodFlex)

%fprintf('Variance of Price Changes, all:              %5.4f\n',PCStats.disppc)
%fprintf('Variance of Price Changes, only changes:     %5.4f\n',PCStats.disppce)
%fprintf('Variance of Real Prices, all:                %5.4f\n',PCStats.disprp)
%fprintf('Variance of Real Prices, only changes:       %5.4f\n',PCStats.disprpe)
%fprintf('Skewness of Price Changes, all:              %5.3f\n',PCStats.skewpc)
%fprintf('Skewness of Price Changes, only changes:     %5.3f\n',PCStats.skewpce)
%fprintf('Skewness of Real Prices, all:                %5.3f\n',PCStats.skewrp)
%fprintf('Skewness of Real Prices, only changes:       %5.3f\n',PCStats.skewrpe)
fprintf('\n')

% Save Model Input and Output
%**************************************************************

% Inputs
GE1Model.a = a;
GE1Model.rp = rp;
GE1Model.rad = rad;
GE1Model.K = K;
GE1Model.beta = beta;
GE1Model.theta = theta;
GE1Model.gamma = gamma;
GE1Model.omega = omega;
GE1Model.omega2 = omega2;
GE1Model.rho = rho;
GE1Model.sigma_eps = sigma_eps;
GE1Model.mu = mu;
GE1Model.sigma_eta = sigma_eta;

% Outputs
GE1Model.G = G;
GE1Model.rV = rV;
GE1Model.F = F;
GE1Model.F_c = F_c;
GE1Model.Q = Q;
GE1Model.GError = GError;
GE1Model.freq = PCStats.freq;
GE1Model.fracup = PCStats.fracup;
GE1Model.sizepc = PCStats.abssizepc;
GE1Model.sizeup = PCStats.sizeup;
GE1Model.sizedw = PCStats.sizedw;
GE1Model.CstdLogOutput = PCStats.stdLogOutput;

save('output_codes/GE1Model','GE1Model')

figure(4)
Q = reshape(Q,[agridsize rpgridsize radgridsize]);
% sum(sum(Q))
p1 = surf(a,rp,Q(:,:,floor(radgridsize/2))');
% p1 = surf(a,rp,Q(:,:,5)');

toc
save('output_codes/basic','GE1Model')
%SimboG1S